package cn.iocoder.yudao.module.ecw.controller.admin.region;

import cn.iocoder.yudao.framework.apollo.core.event.ChannelDataEvent;
import cn.iocoder.yudao.framework.apollo.core.event.QueryChannelInfoEvent;
import org.apache.commons.lang3.StringUtils;
import org.springframework.context.ApplicationContext;
import org.springframework.web.bind.annotation.*;
import javax.annotation.Resource;
import org.springframework.validation.annotation.Validated;
import org.springframework.security.access.prepost.PreAuthorize;
import io.swagger.annotations.*;

import javax.validation.*;
import javax.servlet.http.*;
import java.util.*;
import java.io.IOException;
import java.util.stream.Collectors;

import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.framework.common.pojo.CommonResult;

import static cn.iocoder.yudao.framework.common.exception.enums.GlobalErrorCodeConstants.INTERNAL_SERVER_ERROR;
import static cn.iocoder.yudao.framework.common.pojo.CommonResult.error;
import static cn.iocoder.yudao.framework.common.pojo.CommonResult.success;

import cn.iocoder.yudao.framework.excel.util.ExcelUtils;

import cn.iocoder.yudao.framework.operatelog.core.annotations.OperateLog;
import static cn.iocoder.yudao.framework.operatelog.core.enums.OperateTypeEnum.*;
import static cn.iocoder.yudao.module.ecw.enums.ErrorCodeConstants.*;

import cn.iocoder.yudao.module.ecw.controller.admin.region.vo.*;
import cn.iocoder.yudao.module.ecw.dal.dataobject.region.RegionDO;
import cn.iocoder.yudao.module.ecw.convert.region.RegionConvert;
import cn.iocoder.yudao.module.ecw.service.region.RegionService;

@Api(tags = "管理后台 - 国家城市管理")
@RestController
@RequestMapping("/ecw/region")
@Validated
public class RegionController {

    @Resource
    private RegionService regionService;
    @Resource
    private ApplicationContext applicationContext;

    @PostMapping("/create")
    @ApiOperation("创建区域设置")
    @PreAuthorize("@ss.hasPermission('ecw:region:create')")
    public CommonResult<Long> createRegion(@Valid @RequestBody RegionCreateReqVO createReqVO) {
        return success(regionService.createRegion(createReqVO));
    }

    @PutMapping("/update")
    @ApiOperation("更新区域设置")
    @PreAuthorize("@ss.hasPermission('ecw:region:update')")
    public CommonResult<Boolean> updateRegion(@Valid @RequestBody RegionUpdateReqVO updateReqVO) {
        regionService.updateRegion(updateReqVO);
        return success(true);
    }

    @PutMapping("/updateStatus")
    @ApiOperation("更新区域状态设置")
    @PreAuthorize("@ss.hasPermission('ecw:region:update')")
    public CommonResult<Boolean> updateStatus(@Valid @RequestBody RegionUpdateStatusReqVO updateReqVO) {
        regionService.updateRegionStatus(updateReqVO);
        return success(true);
    }

    @DeleteMapping("/delete")
    @ApiOperation("删除区域设置")
    @ApiImplicitParam(name = "id", value = "编号", required = true, example = "1024", dataTypeClass = Long.class)
    @PreAuthorize("@ss.hasPermission('ecw:region:delete')")
    public CommonResult<Boolean> deleteRegion(@RequestParam("id") Long id) {
        regionService.deleteRegion(id);
        return success(true);
    }

    @GetMapping("/get")
    @ApiOperation("获得区域设置")
    @ApiImplicitParam(name = "id", value = "编号", required = true, example = "1024", dataTypeClass = Long.class)
//    @PreAuthorize("@ss.hasPermission('ecw:region:query')")
    public CommonResult<RegionRespVO> getRegion(@RequestParam("id") Long id) {
        RegionDO region = regionService.getRegion(id);
        return success(RegionConvert.INSTANCE.convert(region));
    }

    @GetMapping("/listByIds")
    @ApiOperation("获得区域设置列表")
    @ApiImplicitParam(name = "ids", value = "编号列表", required = true, example = "1024,2048", dataTypeClass = List.class)
//    @PreAuthorize("@ss.hasPermission('ecw:region:query')")
    public CommonResult<List<RegionRespVO>> getRegionList(@RequestParam("ids") Collection<Long> ids) {
        List<RegionDO> list = regionService.getRegionList(ids);
        return success(RegionConvert.INSTANCE.convertList(list));
    }

    @GetMapping("/list")
    @ApiOperation("获得区域设置列表")
//    @PreAuthorize("@ss.hasPermission('ecw:region:query')")
    public CommonResult<List<RegionRespVO>> getRegionList(RegionListReqVO reqVO) {
        List<RegionDO> list = regionService.getSimpleDepts(reqVO);
        list.sort(Comparator.comparing(RegionDO::getSort));
        return success(RegionConvert.INSTANCE.convertList(list));
    }

    @GetMapping("/page")
    @ApiOperation("获得区域设置分页")
//    @PreAuthorize("@ss.hasPermission('ecw:region:query')")
    public CommonResult<PageResult<RegionRespVO>> getRegionPage(@Valid RegionPageReqVO pageVO) {
        PageResult<RegionDO> pageResult = regionService.getRegionPage(pageVO);
        return success(RegionConvert.INSTANCE.convertPage(pageResult));
    }

    @GetMapping("/export-excel")
    @ApiOperation("导出区域设置 Excel")
    @PreAuthorize("@ss.hasPermission('ecw:region:export')")
    @OperateLog(type = EXPORT)
    public void exportRegionExcel(@Valid RegionExportReqVO exportReqVO,
              HttpServletResponse response) throws IOException {
        List<RegionDO> list = regionService.getRegionList(exportReqVO);
        // 导出 Excel
        List<RegionExcelVO> datas = RegionConvert.INSTANCE.convertList02(list);
        ExcelUtils.write(response, "区域设置.xls", "数据", RegionExcelVO.class, datas);
    }

    @GetMapping("/getTree")
    @ApiOperation("获得区域子列表")
    @ApiImplicitParam(name = "id", value = "编号", required = true, example = "1024", dataTypeClass = Long.class)
//    @PreAuthorize("@ss.hasPermission('ecw:region:query')")
    public CommonResult<RegionRespVO> getTree(@RequestParam("id") Long id) {
        List<RegionDO> list = regionService.getByParent(id);
        List<RegionRespVO> dataList = RegionConvert.INSTANCE.convertList(list);
        Optional<RegionRespVO> optional = dataList.stream().filter(t -> t.getId().equals(id)).findFirst();
        if(!optional.isPresent()) {
            return error(INTERNAL_SERVER_ERROR.getCode(), "数据不存在");
        }
        RegionRespVO regionRespVO =  optional.get();
        buildChildTree(regionRespVO, dataList);
        return success(regionRespVO);
    }

    @GetMapping("/getListTree")
    @ApiOperation("获得区域设置列表")
    //@PreAuthorize("@ss.hasPermission('ecw:region:query')")
    public CommonResult<List<RegionRespVO>> listTree(RegionListReqVO reqVO) {
        List<RegionDO> list = regionService.getSimpleDepts(reqVO);
        list.sort(Comparator.comparing(RegionDO::getSort));
        List<RegionRespVO> dataList = RegionConvert.INSTANCE.convertList(list);

        List<RegionRespVO> continents = dataList.stream()
                .filter(t -> t.getParentId() == 0)
                .collect(Collectors.toList());
        for (RegionRespVO regionRespVO : continents) {
            buildChildTree(regionRespVO, dataList);
        }
        if(reqVO.getTreeType() == 0) {
            return success(continents);
        }
        List<RegionRespVO> countryList = continents.stream()
                .map(RegionRespVO::getChildren)
                .flatMap(Collection::stream)
                .collect(Collectors.toList());

        return success(countryList);
    }

    @GetMapping("/getCityList")
    @ApiOperation("获得区域城市列表")
//    @PreAuthorize("@ss.hasPermission('ecw:region:query')")
    public CommonResult<List<RegionRespVO>> getCityList() {
        List<RegionDO> cityList = regionService.getCityList();
        return success(RegionConvert.INSTANCE.convertList(cityList));
    }

    @GetMapping("/getTradeCityList")
    @ApiOperation("获得始发、目的城市列表")
    //@PreAuthorize("@ss.hasPermission('ecw:region:query')")
    public CommonResult<List<RegionRespVO>> getTradeCityList(
            TradeRegionListReqVO reqVO) {

        if (Objects.nonNull(reqVO.getChannelId())){
            // 如果有渠道信息，则查询可出目的地国家下的目的地城市
            QueryChannelInfoEvent channelInfoEvent = new QueryChannelInfoEvent();
            channelInfoEvent.setChannelId(reqVO.getChannelId());
            applicationContext.publishEvent(channelInfoEvent);
            if (Objects.nonNull(channelInfoEvent.getCountryId()) && channelInfoEvent.getCountryId() > 0L){
                reqVO.setDestCountryId(channelInfoEvent.getCountryId());
            }
        }
        List<RegionDO> cityList = regionService.getImportExportCityList(reqVO);
        return success(RegionConvert.INSTANCE.convertList(cityList));
    }

    @GetMapping("/getTradeCountryList")
    @ApiOperation("获得目的国家列表")
    public CommonResult<List<RegionRespVO>> getTradeCountryList() {
        List<RegionDO> countryList = regionService.getImportExportCountryList();
        return success(RegionConvert.INSTANCE.convertList(countryList));
    }

    private RegionRespVO buildChildTree(RegionRespVO pNode, List<RegionRespVO> allDatas){
        List<RegionRespVO> childs = new  ArrayList<>();
        for(RegionRespVO itemNode : allDatas) {
            if(itemNode.getParentId().equals(pNode.getId())) {
                childs.add(buildChildTree(itemNode, allDatas));
            }
        }
        pNode.setChildren(childs);
        return pNode;
    }

    @GetMapping("/getCityListByParent")
    @ApiOperation("根据父节点获取城市列表")
    public CommonResult<List<RegionRespVO>> getCityListByParent(@RequestParam("id") Long parentId) {
        if (parentId == null) {
            return CommonResult.success(Collections.emptyList());
        }
        List<RegionDO> cityList = regionService.getCityListByParentId(parentId);
        return success(RegionConvert.INSTANCE.convertList(cityList));
    }


    @GetMapping("/check/dest-currency/area-code/")
    @ApiOperation("检查国家区域是否与手机号国家区号一致")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "currencyId", value = "国家ID", required = true, example = "1024", dataTypeClass = Long.class),
            @ApiImplicitParam(name = "areaCode", value = "手机国家区号", required = true, example = "86", dataTypeClass = String.class)
    })
    public CommonResult<Boolean> checkDestCurrencyAndAreaCode(@RequestParam("currencyId") Long currencyId,
                                                              @RequestParam("areaCode") String areaCode) {
        if (currencyId == null) {
            return CommonResult.error(CURRENCY_ID_NOT_NULL);
        }
        if (StringUtils.isBlank(areaCode)) {
            return CommonResult.error(AREA_CODE_NOT_NULL);
        }
        return success(regionService.countByIdAndAreaCode(currencyId, areaCode));
    }
}
